### Application to the suicide attempter data, Just, et al. (2017)
#### suicide attempter data has 180 obs. with 24547 variables.
#### More application details available in Dai, Dutta and Maitra (2019).

#### Required packages:
##### fad: See install instructions in install.R.
##### doMC: Foreach parallel adaptor for 'parallel'.

## Load packages and source codes.
library("fad") 
library("doMC")
registerDoMC(16) # Parelled with 16 cores.

source("fapc.R") # Initialization for EM algorithm.
source("faem.R") # EM algorithm.

## Load dataset
load("suicide.rda") # Data available in Just et al.(2017)

## Save multiple outputs of foreach dopar loop
multiResultClass <- function(r1=NULL,t1=NULL,r2=NULL,t2=NULL)
{
  m <- list(r1 = r1, t1 = t1, r2 = r2, t2 = t2)
  ## Set the name for the class
  class(m) <- append(class(m),"multiResultClass")
  return(m)
}

## Obtain data matrix from the suicide attempters
### Remove the last column which contains the group information
X1 <- df1.s[df1.s$class == "attempter:n",-1]
X2 <- df1.s[df1.s$class == "attempter:s",-1]
X <- rbind(X1,X2)
X <- as.matrix(X)

## Run for number of factors q from 1-10
### Note: when q = 0, parameter estimates are sample means and sample variances.
RES <- foreach(q = 1:10) %dopar% {
  result <- multiResultClass()
  
  ## FAD
  ### For stopping criteria used in Dai, Dutta and Maitra (2019):
  ### factr = 100 in optim_share function (default in fad)   
  ptm <- proc.time()
  res1 <- fad(x = X,factors = q)
  time <- proc.time() - ptm
  t1 <- time[3]
  r1 <- res1
  
  ## EM
  ### For comparable stopping criteria with FAD:
  ### Control the tolerance parameter for gradient 
  ### Note: Maximum number of iterations = 5000
  ptm <- proc.time()
  res2 <- fa.em(Y = X,r = q,gtol = res1$gerr)
  time <- proc.time() - ptm
  t2 <- time[3]
  r2 <- res2
  
  ## Collect results for each factor model
  result$r1 <- r1
  result$t1 <- t1
  result$r2 <- r2
  result$t2 <- t2
  return(result)
}

# Save results
save(RES, file = "app-suicide-a.rda")

