### Initialization for EM algorithm for factor model of Gaussian data
#### Values returned for scaled data matrix

#### Function is adapted from the R cate package, Wang and Zhao (2018)
#### Reference: https://cran.r-project.org/web/packages/cate/

#### Factor loadings: Gamma and uniqueness: Sigma are initialized from 
#### the principal component analysis results via the the Lanczos algorithm

#### Required packages:
##### RSpectra: Partial SVD using Lanczos algorithm

## Load packages 
library(RSpectra)

fa.pc <- compiler::cmpfun(function(Y, r) {
  Y <- scale(Y, scale = T)
  svd.Y <- RSpectra::svds(Y, k = r)
  
  Gamma <- svd.Y$v %*% diag(svd.Y$d, r, r) / sqrt(nrow(Y))
  Z <- sqrt(nrow(Y)) * svd.Y$u
  
  Sigma <- apply(Y - Z %*% t(Gamma), 2, function(x) mean(x^2))
  
  return(list(Gamma = Gamma,
              Z = Z,
              Sigma = Sigma))
  
})
